-- Pin by Jadya
-- EU - Well of Eternity

local tsort = table.sort

local addonName, addonTable = ...
local L = addonTable.L

local ng = NyxGUI("1.0")

local settings = addonTable.settings
local notes    = addonTable.NoteFunctions
local groups   = addonTable.GroupFunctions

local initialized, mf, tabframe, sel_group, sel_pin, classnames
local editbox
local selected_group, selected_pin
local groupslist = {}
local pinslist   = {}

local function cancel_editbox_changes()
 if not editbox.editing then return end
 
 editbox.editing.name = editbox.lastname
 editbox.editing = nil
 editbox.lastname = ""
 editbox:Hide()
 PinMainframeGroupsList:update()
end

local function save_editbox_changes()
 if not editbox.editing then return end
 
 editbox.editing.name = editbox:GetText()
 editbox.editing = nil
 editbox.lastname = ""
 editbox:Hide()
 PinMainframeGroupsList:update()
end

function settings:ShowEditBox(button)
 if button.wasSelected then
  save_editbox_changes()
  editbox.editing = groupslist[button.index]
  editbox.lastname = editbox.editing.name
  editbox:SetParent(button)
  editbox:SetPoint("TOPLEFT", button, "TOPLEFT")
  editbox:SetText(editbox.editing.name)
  editbox:Show()
  editbox:SetFocus()
 end
end

local function update_group_widgets()
 if not selected_group then
  PinMainframePinsAdd:Hide()
  PinMainframePinsRemove:Hide()
  --PinMainframePinsClone:Hide()
  
  if mf.btn_visibility.selected then
   mf.btn_pins:Click()
  end
  mf.btn_visibility:Hide()
  
  return
 end

 if selected_group.visibility == 2 then
  _G["PinSettings_visibility_char_"..addonTable.playername]:SetChecked(selected_group["visibility_char_"..addonTable.playername] or false)

  for _,v in pairs(addonTable.classes) do
   _G["PinSettings_visibility_class_"..v]:SetChecked(selected_group["visibility_class_"..v] or false)
  end
  mf.btn_visibility:Show()
 else
  if mf.btn_visibility.selected then
   mf.btn_pins:Click()
  end
  mf.btn_visibility:Hide()  
 end

 PinMainframePinsAdd:Show()

 if not selected_pin then
  PinMainframePinsRemove:Hide()
  --PinMainframePinsClone:Hide()
  --PinMainframePinsEdit:Hide()
  return
 end
 
 PinMainframePinsRemove:Show()
 --PinMainframePinsClone:Show()
 --PinMainframePinsEdit:Show()
end

function settings:CycleGroupVisibilityByListIndex(i)
 local g = groupslist[i]
 if not g then return end
 
 if g.visibility == 2 then
  g.visibility = 0
 else
  g.visibility = (g.visibility or -1) + 1
 end

 PinMainframeGroupsList.sel_index = i

 PinMainframeGroupsList:update()
 update_group_widgets()
end

local function group_selected(group)
 if group == selected_group then return end
 
 selected_group = group
 selected_pin = nil
 PinMainframePinsList.sel_index = nil
 save_editbox_changes()
 update_group_widgets()
 
 for _,v in pairs(PinSaves.groups) do
  groups:Hide(v)
 end
 
 PinMainframePinsList.list = selected_group and selected_group.notes or nil
 groups:Show(group)
end

local function pin_selected(pin)
 if pin == selected_pin then return end
 
 selected_pin = pin
 
 update_group_widgets()
end

------------ Button Updates ------------
local function button_update_groups(btn, list, index)
 local g = list[index]
 
 btn.name:SetText(g.name or "<no name>")
 btn.contents:SetText(L["notes"]..": #"..#(g.notes))

 if g.visibility == 0 then
  btn.btnvisibility:SetText(SHOW)
  btn.btnvisibility:SetBackdropColor(0, 1, 0, 0.1)
  btn.btnvisibility:SetBackdropBorderColor(0, 1, 0, 1)
 elseif g.visibility == 1 then
  btn.btnvisibility:SetText(HIDE)
  btn.btnvisibility:SetBackdropColor(1, 0, 0, 0.1)
  btn.btnvisibility:SetBackdropBorderColor(1, 0, 0, 1)
 else
  btn.btnvisibility:SetText(L["custom"])
  btn.btnvisibility:SetBackdropColor(1, 1, 0, 0.1)
  btn.btnvisibility:SetBackdropBorderColor(1, 1, 0, 1)
 end
 
 local l = btn:GetParent():GetParent()

 if index == l.sel_index then
  group_selected(g)
 end
end

local function button_update_pins(btn, list, index)
 local l = btn:GetParent():GetParent()

 if index == l.sel_index then
  pin_selected(PinSaves.notes[list[index]])
  l.sel_tag = list[index]
 end

 btn.pin = addonTable.notes[list[index]].pin
 btn:SetBackdropColor(addonTable.notes[list[index]].pin:GetBackdropColor())
 btn:SetBackdropBorderColor(addonTable.notes[list[index]].pin:GetBackdropBorderColor())

 --btn.name:SetText(addonTable.notes[list[index]].name)
 --btn.text:SetText(addonTable.notes[list[index]].text:GetText())
end
----------------------------------------

------------ Listbox Updates ------------
local function listbox_pins_update_pre(self)
 local sel = PinMainframeGroupsList.sel_index
 wipe(pinslist)
 
 if sel and groupslist[sel] then
  for k,v in pairs(groupslist[sel].notes) do
   pinslist[k] = v
  end
 end
end

local function listbox_groups_update_pre(self)
 wipe(groupslist)
 for k,v in pairs(PinSaves.groups) do
  tinsert(groupslist, v)
 end
 tsort(groupslist, function(a,b) return a.id < b.id end)
end

local function listbox_groups_update_post(self)
 PinMainframePinsList:update()
 
 if not PinMainframeGroupsList.sel_index or not groupslist[PinMainframeGroupsList.sel_index] then
  group_selected(nil)
  PinMainframePinsAdd:Hide()
 else
  PinMainframePinsAdd:Show()
 end
end
-----------------------------------------

local function add_group()
 local l = PinMainframeGroupsList

 local group = groups:New()
 
 l.sel_index = #groupslist + 1
 l:update()
end

local function delete_group()
 if not selected_group then return end
 
 local l = PinMainframeGroupsList
 
 groups:Delete(selected_group)

 l.sel_index = math.max(1, l.sel_index - 1)
 l:update()
end

local function setvar_group(self)
 if not selected_group then return end
 selected_group[self.key] = self.tag
end

local function setbool_group(self)
 if not selected_group then return end
 selected_group[self.key] = self:GetChecked() or nil
end

local function add_pin()
 if not selected_group then return end
 
 local l = PinMainframePinsList

 local id = notes:New()
 tinsert(selected_group.notes, id)
 l.sel_index = #pinslist + 1
 PinMainframeGroupsList:update()
end

local function delete_pin()
 if not selected_pin then return end

 local l = PinMainframePinsList

 table.remove(selected_group.notes, l.sel_index)
 notes:Delete(addonTable.notes[l.sel_tag])
 l.sel_index = math.max(1, l.sel_index - 1)
 PinMainframeGroupsList:update()
 update_group_widgets()
end

local function clone_pin()
 --
end

--[[
local function edit_pin()
 local l = PinMainframePinsList
 
 if l.sel_index == nil or not PinSaves.notes[l.sel_tag] then return end

 settings:EditNote(addonTable.notes[l.sel_tag])
end
]]

local function Initialize()

 local f

 classnames = {}
 FillLocalizedClassList(classnames)
 classnames["DEMONHUNTER"] = "Demon Hunter" -- temporary

 mf = ng:New("Pin", "Frame", "PinMainframe", UIParent)
 mf:SetFrameStrata("HIGH")
 mf:EnableMouse(true)
 mf:SetMovable(true)
 mf:RegisterForDrag("LeftButton")
 mf:SetScript("OnDragStart", function(self) self:StartMoving() end)
 mf:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
 mf:SetSize(500, 500)
 mf:SetPoint("CENTER")
 mf:SetScript("OnShow", function()
   PinMainframeGroupsList:update()
  end)
 
 mf.logo = mf:CreateTexture()
 mf.logo:SetPoint("TOPRIGHT")
 mf.logo:SetSize(130,100)
 mf.logo:SetTexture("Interface\\Addons\\Pin\\logo.tga")
 
 mf:SetScript("OnHide", groups.ShowActive)
 
 editbox = ng:New("Pin", "Editbox", "PinMainframeEditbox", mf)
 editbox:SetFrameStrata("HIGH")
 editbox:SetSize(100, 15)
 editbox.func = save_editbox_changes
 editbox.cancelfunc = cancel_editbox_changes
 
 local g1 = ng:New("Pin", "Groupbox", nil, mf)
 g1:SetSize(170,400)
 g1:SetPoint("TOPLEFT", mf, "TOPLEFT", 10, -60)
 g1:SetText(GROUPS)

 -- groups listbox 
 f = ng:New("Pin", "Listbox", "PinMainframeGroupsList", g1)
 f:SetAllPoints()

 tabframe = ng:New("Pin", "Groupbox", nil, mf)
 tabframe:SetPoint("TOPLEFT", PinMainframeGroupsList, "TOPRIGHT", 40, -20)
 tabframe:SetPoint("BOTTOMLEFT", PinMainframeGroupsList, "BOTTOMRIGHT", 40, 20)
 tabframe:SetWidth(270)

 -- groups add/del buttons
 f = ng:New("Pin", "Button", nil, mf)
 f:SetSize(20, 20)
 f:SetPoint("TOPRIGHT", PinMainframeGroupsList, "TOPRIGHT", -25, 25)
 f:SetText("+")
 f:SetScript("OnClick", add_group)

 f = ng:New("Pin", "Button", nil, mf)
 f:SetSize(20, 20)
 f:SetPoint("TOPRIGHT", PinMainframeGroupsList, "TOPRIGHT", 0, 25)
 f:SetText("-")
 f:SetScript("OnClick", delete_group)

 -- pins tab
 mf.btn_pins = ng:New("Pin", "Tabbutton", nil, tabframe)
 mf.btn_pins:SetText(L["notes"])
 local tab = mf.btn_pins.tab
 
 f = ng:New("Pin", "Listbox", "PinMainframePinsList", tab)
 f:SetPoint("TOPLEFT")
 f:SetPoint("BOTTOMRIGHT", tab, "BOTTOMLEFT", 150, 0)
 
 -- pin buttons
 f = ng:New("Pin", "Button", "PinMainframePinsAdd", tab)
 f:SetPoint("LEFT", PinMainframePinsList, "RIGHT", 30, 45)
 f:SetText(ADD)
 f:SetScript("OnClick", add_pin)
 
 f = ng:New("Pin", "Button", "PinMainframePinsRemove", tab)
 f:SetPoint("LEFT", PinMainframePinsList, "RIGHT", 30, 15)
 f:SetText(REMOVE)
 f:SetScript("OnClick", delete_pin)

 --[[f = CreateFrame("Button", "PinMainframePinsClone", tab, "SaregonButton")
 f:SetPoint("LEFT", PinMainframePinsList, "RIGHT", 30, -15)
 f:SetText(L["clone"])
 f:SetScript("OnClick", clone_pin)
 ]]
--[[
 f = CreateFrame("Button", "PinMainframePinsEdit", tab, "SaregonButton")
 f:SetPoint("LEFT", PinMainframePinsList, "RIGHT", 30, -45)
 f:SetText(EDIT)
 f:SetScript("OnClick", edit_pin)
 ]]

 -- visibility tab
 mf.btn_visibility = ng:New("Pin", "Tabbutton", nil, tabframe)
 mf.btn_visibility:SetText(L["visibility"])
 tab = mf.btn_visibility.tab

 local function radiobutton(p, text, offsetY, key, value, func)
  local result = ng:New("Pin", "Radiobutton", "PinSettings_"..key..value, p)
  result:SetPoint("TOPLEFT", p, "TOPLEFT", 0, offsetY)
  result:SetText(text)
  result.key = key
  result.tag = value
  result:SetScript("OnClick", func)
 end

 local function checkbutton(p, text, offsetY, key, func)
 local result = ng:New("Pin", "Checkbox", "PinSettings_"..key, p)
  result:SetPoint("TOPLEFT", p, "TOPLEFT", 0, offsetY)
  result:SetText(text)
  result.key = key
  result:SetScript("OnClick", func)
 end

 g1 = ng:New("Pin", "Groupbox", nil, tab)
 g1:SetText(L["available_to"])
 g1:SetPoint("TOPLEFT", tab, "TOPLEFT", 5, -15)
 g1:SetSize(110, 280)

 checkbutton(g1, L["this_char"], 0, "visibility_char_"..addonTable.playername, setbool_group)

 checkbutton(g1, classnames["WARRIOR"],       -30, "visibility_class_WARRIOR",     setbool_group)
 checkbutton(g1, classnames["ROGUE"],         -50, "visibility_class_ROGUE",       setbool_group)
 checkbutton(g1, classnames["MAGE"],          -70, "visibility_class_MAGE",        setbool_group)
 checkbutton(g1, classnames["PRIEST"],        -90, "visibility_class_PRIEST",      setbool_group)
 checkbutton(g1, classnames["WARLOCK"],      -110, "visibility_class_WARLOCK",     setbool_group)
 checkbutton(g1, classnames["HUNTER"],       -130, "visibility_class_HUNTER",      setbool_group)
 checkbutton(g1, classnames["SHAMAN"],       -150, "visibility_class_SHAMAN",      setbool_group)
 checkbutton(g1, classnames["DRUID"],        -170, "visibility_class_DRUID",       setbool_group)
 checkbutton(g1, classnames["MONK"],         -190, "visibility_class_MONK",        setbool_group)
 checkbutton(g1, classnames["PALADIN"],      -210, "visibility_class_PALADIN",     setbool_group)
 checkbutton(g1, classnames["DEATHKNIGHT"],  -230, "visibility_class_DEATHKNIGHT", setbool_group)
 checkbutton(g1, classnames["DEMONHUNTER"],  -250, "visibility_class_DEMONHUNTER", setbool_group)

 -- close button
 f = ng:New("Pin", "Button", nil, mf)
 f:SetPoint("BOTTOM", mf, "BOTTOM", 0, 10)
 f:SetText(CLOSE)
 f:SetScript("OnClick", function()
   mf:Hide()
  end)

 mf.btn_pins:Click()
 
 PinMainframePinsList.update_pre = listbox_pins_update_pre
 PinMainframePinsList.update_button = button_update_pins
 PinMainframePinsList:Initialize("ListboxPinsButton", pinslist)
 
 PinMainframeGroupsList.update_pre = listbox_groups_update_pre
 PinMainframeGroupsList.update_post = listbox_groups_update_post
 PinMainframeGroupsList.update_button = button_update_groups
 PinMainframeGroupsList:Initialize("ListboxGroupsButton", groupslist)
 
 update_group_widgets()
 
 initialized = true
end

function settings:Mainframe()
 if not initialized then 
  Initialize() 
 elseif mf:IsVisible() then
  mf:Hide()
 else
  if selected_group then
   selected_group = nil
   group_selected(selected_group)
  end
  mf:Show() 
 end
end