-- Pin by Jadya
-- EU - Well of Eternity

local floor = math.floor

local addonName, addonTable = ...
local L = addonTable.L

local ng = NyxGUI("1.0")

local settings = addonTable.settings
local notes = addonTable.NoteFunctions
local reminder = addonTable.ReminderFunctions

local initialized, mf, editing_note

local function save_colors()
 notes:SaveColors(editing_note)
end

local function reminder_time_refresh(h, m)
 if not h then h = floor(PinSettingsReminderSetHour:GetValue()) end
 if not m then m = floor(PinSettingsReminderSetMinute:GetValue()) end
 PinSettingsReminderHour:SetText(((h < 10) and ("0"..h) or h)..":"..((m < 10) and ("0"..m) or m))
end

local function reminder_time_set_hour()
 local h = floor(PinSettingsReminderSetHour:GetValue())
 notes:SetAttr(editing_note, "reminder_hour", h)
 reminder_time_refresh(h)
end

local function reminder_time_set_minute()
 local m = floor(PinSettingsReminderSetMinute:GetValue())
 notes:SetAttr(editing_note, "reminder_minute", m)
 reminder_time_refresh(nil, m)
end

local function reminder_day_refresh()
 if not editing_note.reminder_date then
  local _, m , d, y = CalendarGetDate()
  notes:SetAttr(editing_note, "reminder_date", {m, d, y})
 end
  
 local month, day, year = unpack(editing_note.reminder_date)

 PinSettingsReminderDay:SetText(day.."/"..month.."/"..year)
end

function settings:SetReminderDate(month, day, year)
 notes:SetAttr(editing_note, "reminder_date", {month, day, year})
 reminder_day_refresh()
end

local function reminder_type_tab_click(self)
 notes:SetAttr(editing_note, "reminder_type", self.tag)
end

local function reminder_tab_refresh()
 if editing_note.reminder_enabled then
  mf.btn_reminder:SetBackdropColor(0, 0.3, 0, 1) 
 else
  mf.btn_reminder:SetBackdropColor(0.3, 0, 0, 1)
 end
end

local function Initialize()

 local f, p

 mf = ng:New("Pin", "Frame", "PinSettingsFrame", UIParent)
 mf:SetFrameStrata("DIALOG")
 mf:EnableMouse(true)
 mf:SetMovable(true)
 mf:RegisterForDrag("LeftButton")
 mf:SetScript("OnDragStart", function(self) self:StartMoving() end)
 mf:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
 mf:SetSize(550, 310)
 mf:SetPoint("CENTER")
 
 mf.logo = mf:CreateTexture()
 mf.logo:SetPoint("TOPRIGHT")
 mf.logo:SetSize(130,100)
 mf.logo:SetTexture("Interface\\Addons\\Pin\\logo.tga")
 
 -- tabframe
 f = ng:New("Pin", "Groupbox", nil, mf, false)
 f:SetBackdropColor(0, 0, 0, 0)
 f:SetBackdropBorderColor(0, 0, 0, 0)
 f:SetPoint("TOPLEFT", mf, "TOPLEFT", 2, -60)
 f:SetPoint("BOTTOMRIGHT", mf, "BOTTOMRIGHT", -2)
 mf.tabframe = f

 -- appearance
 mf.btn_appearance = ng:New("Pin", "Tabbutton", nil, mf.tabframe)
 mf.btn_appearance:SetText(APPEARANCE_LABEL)
 tab = mf.btn_appearance.tab
 
 -- appearance->note
 local g1 = ng:New("Pin", "Groupbox", nil, tab)
 g1:SetSize(180,180)
 g1:SetPoint("TOPLEFT", tab, "TOPLEFT", 2, -20)
 g1:SetText(L["note"])

 f = ng:New("Pin", "Colorbutton", "PinSettingsAppearanceNoteBG", g1)
 f:SetSize(20, 20)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT", 5, -5)
 f.mode = "backdrop"
 f.func = save_colors
 f:SetText(L["background_color"])
 
 f = ng:New("Pin", "Colorbutton", "PinSettingsAppearanceNoteBorder", g1)
 f:SetSize(20, 20)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT", 5, -30)
 f.mode = "backdrop_border"
 f.func = save_colors
 f:SetText(EMBLEM_BORDER_COLOR)

 f = ng:New("Pin", "SliderH", "PinSettingsAppearanceNoteWidth", g1)
 f:SetPoint("TOP", g1, "TOP", 0, -65)
 f:SetText(COMPACT_UNIT_FRAME_PROFILE_FRAMEWIDTH)
 f:SetMinMaxValues(90, 350)
 f:SetValue(300)
 f.onValueChanged = function(self)
                     notes:SetMaxWidth(editing_note, self:GetValue())
                    end

 -- appearance->pin
 local g2 = ng:New("Pin", "Groupbox", nil, tab)
 g2:SetSize(180,180)
 g2:SetPoint("LEFT", g1, "RIGHT", 1, 0)
 g2:SetText(L["pin"])

 f = ng:New("Pin", "Colorbutton", "PinSettingsAppearancePinBG", g2)
 f:SetSize(20, 20)
 f:SetPoint("TOPLEFT", g2, "TOPLEFT", 5, -5)
 f.mode = "backdrop"
 f:SetText(L["background_color"])
 f.func = save_colors
 
 f = ng:New("Pin", "Colorbutton", "PinSettingsAppearancePinBorder", g2)
 f:SetSize(20, 20)
 f:SetPoint("TOPLEFT", g2, "TOPLEFT", 5, -30)
 f.mode = "backdrop_border"
 f:SetText(EMBLEM_BORDER_COLOR)
 f.func = save_colors

 f = ng:New("Pin", "SliderH", "PinSettingsAppearancePinWidth", g1)
 f:SetPoint("TOP", g2, "TOP", 0, -65)
 f:SetText(COMPACT_UNIT_FRAME_PROFILE_FRAMEWIDTH)
 f:SetMinMaxValues(20, 130)
 f:SetValue(200)
 f.onValueChanged = function(self)
                     notes:SetPinWidth(editing_note, self:GetValue())
                    end

 f = ng:New("Pin", "SliderH", "PinSettingsAppearancePinHeight", g1)
 f:SetPoint("TOP", g2, "TOP", 0, -105)
 f:SetText(COMPACT_UNIT_FRAME_PROFILE_FRAMEHEIGHT)
 f:SetMinMaxValues(20, 130)
 f:SetValue(100)
 f.onValueChanged = function(self)
                     notes:SetPinHeight(editing_note, self:GetValue())
                    end

 -- appearance->font
 local g3 = ng:New("Pin", "Groupbox", nil, tab)
 g3:SetSize(180,180)
 g3:SetPoint("LEFT", g2, "RIGHT", 1, 0)
 g3:SetText(L["font"])
 
 f = ng:New("Pin", "Colorbutton", "PinSettingsAppearanceFontColor", g3)
 f:SetSize(20, 20)
 f:SetPoint("TOPLEFT", g3, "TOPLEFT", 5, -5)
 f.mode = "fontstring"
 f:SetText(COLOR)
 f.func = save_colors
 
 f = ng:New("Pin", "SliderH", "PinSettingsAppearanceFontSize", g3)
 f:SetPoint("TOP", g3, "TOP", 0, -65)
 f:SetText(L["size"])
 f:SetMinMaxValues(4, 30)
 f:SetValue(12)
 f.onValueChanged = function(self)
				     notes:SetFont(editing_note, "Fonts\\FRIZQT__.TTF", self:GetValue())
                    end

 -- general
 mf.btn_general = ng:New("Pin", "Tabbutton", nil, mf.tabframe)
 mf.btn_general:SetText(GENERAL)
 tab = mf.btn_general.tab

 local function setvar_radio(self)
  notes:SetAttr(editing_note, self.key, self.tag)
 end
 
 local function setvar_chk(self)
  notes:SetAttr(editing_note, self.key, self:GetChecked() or nil)
  if self == PinSettingsReminderEnabled then
   reminder_tab_refresh()
  end
 end
 
 -- general->combat_behavior
 g1 = ng:New("Pin", "Groupbox", nil, tab)
 g1:SetSize(170,70)
 g1:SetPoint("TOPLEFT", tab, "TOPLEFT", 2, -20)
 g1:SetText(L["combat_behavior"])

 f = ng:New("Pin", "Radiobutton", "PinSettingsGeneralCombatShow", g1)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT", 0, 0)
 f:SetText(PLAYER_DIFFICULTY1)
 f.key = "combat_behavior"
 f.tag = 0
 f.func = setvar_radio
 
 f = ng:New("Pin", "Radiobutton", "PinSettingsGeneralCombatHide", g1)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT", 0, -20)
 f:SetText(HIDE)
 f.key = "combat_behavior"
 f.tag = 1
 f.func = setvar_radio

 f = ng:New("Pin", "Radiobutton", "PinSettingsGeneralCombatClickThrough", g1)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT", 0, -40)
 f:SetText(L["click-through"])
 f.key = "combat_behavior"
 f.tag = 2
 f.func = setvar_radio
 
 -- reminder
 mf.btn_reminder = ng:New("Pin", "Tabbutton", nil, mf.tabframe)
 mf.btn_reminder:SetText(L["reminder"])
 tab = mf.btn_reminder.tab

 -- reminder->g1
 g1 = ng:New("Pin", "Groupbox", nil, tab)
 g1:SetPoint("BOTTOMLEFT", tab, "BOTTOMLEFT", 2, 50)
 g1:SetPoint("TOPLEFT", tab, "TOPLEFT", 2, -20)
 g1:SetWidth(250)
 g1:SetText(L["reminder"])

 f = ng:New("Pin", "Checkbox", "PinSettingsReminderEnabled", g1)
 f:SetPoint("TOPLEFT", g1, "TOPLEFT")
 f:SetText(ENABLE)
 f.key = "reminder_enabled"
 f:SetScript("OnClick", setvar_chk)

 PinSettingsReminderHour = g1:CreateFontString()
 PinSettingsReminderHour:SetFont("Fonts\\FRIZQT__.TTF", 30)
 PinSettingsReminderHour:SetPoint("TOP", g1, "TOP", 0, -30)
 PinSettingsReminderHour:SetText("00:00")

 f = ng:New("Pin", "SliderH", "PinSettingsReminderSetHour", g1)
 f:SetPoint("TOP", g1, "TOP", 0, -65)
 f:SetMinMaxValues(0, 23)
 f:SetText(L["hour"])
 f:SetHeight(15)
 f.text_value:Hide()
 f.onValueChanged = reminder_time_set_hour
 
 f = ng:New("Pin", "SliderH", "PinSettingsReminderSetMinute", g1)
 f:SetPoint("TOP", g1, "TOP", 0, -95)
 f:SetMinMaxValues(0, 59)
 f:SetText(L["minute"])
 f:SetHeight(15)
 f.text_value:Hide()
 f.onValueChanged = reminder_time_set_minute

 -- reminder->g2
 g2 = ng:New("Pin", "Groupbox", nil, tab)
 g2:SetPoint("TOPLEFT", g1, "TOPRIGHT")
 g2:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", -2, 50)

 f = ng:New("Pin", "Tabbutton", "PinSettingsReminderOneDay", g2)
 f:SetText(L["single_day"])
 f:HookScript("OnClick", reminder_type_tab_click)
 f.tag = 0
 f:Click()
 tab = f.tab

 PinSettingsReminderDay = tab:CreateFontString()
 PinSettingsReminderDay:SetFont("Fonts\\FRIZQT__.TTF", 30)
 PinSettingsReminderDay:SetPoint("TOP", tab, "TOP", 0, -30)
 
 local isShowingCalendar
 
 f = ng:New("Pin", "Button", "PinSettingsReminderSetDay", tab)
 f:SetPoint("CENTER", tab, "CENTER", 0, -30)
 f:SetText(L["set"])
 f:SetScript("OnClick", function()
  reminder:ShowCalendar()
  isShowingCalendar = true
  mf:Hide()
  isShowingCalendar = nil
 end)
 
 f = ng:New("Pin", "Tabbutton", "PinSettingsReminderRepeat", g2)
 f:SetText(L["repeat"])
 f:HookScript("OnClick", reminder_type_tab_click)
 f.tag = 1
 tab = f.tab

 for i = 1,7 do
  f = ng:New("Pin", "Checkbox", "PinSettingsReminderWeekday"..i, tab)
  f:SetPoint("TOPLEFT", tab, "TOPLEFT", 5, i * -20)
  f:SetText(addonTable.weekdays[i])
  f.key = "reminder_weekday"..i
  f:SetScript("OnClick", setvar_chk)
 end

 -- close button
 f = ng:New("Pin", "Button", nil, mf)
 f:SetPoint("BOTTOM", mf, "BOTTOM", 0, 10)
 f:SetText(CLOSE)
 f:SetScript("OnClick", function()
   mf:Hide()
  end)
  
 mf:SetScript("OnHide", function()
   if not isShowingCalendar then
    reminder:SetReminder(editing_note)
   end
 
   if PinMainframeGroupsList and PinMainframeGroupsList:IsVisible() then
    PinMainframeGroupsList:update()
	return
   end

   if PinMainframePinsList and PinMainframePinsList:IsVisible() then
    PinMainframePinsList:update()
   end
 end)

 mf.btn_appearance:Click()

 initialized = true
end

local function NoteToFrame(note)
 -- appearance
 PinSettingsAppearanceNoteBG.tag = editing_note
 PinSettingsAppearanceNoteBG:update(editing_note:GetBackdropColor())
 
 PinSettingsAppearanceNoteBorder.tag = editing_note 
 PinSettingsAppearanceNoteBorder:update(editing_note:GetBackdropBorderColor())
 
 PinSettingsAppearancePinBG.tag = editing_note.pin 
 PinSettingsAppearancePinBG:update(editing_note.pin:GetBackdropColor())
 
 PinSettingsAppearancePinBorder.tag = editing_note.pin
 PinSettingsAppearancePinBorder:update(editing_note.pin:GetBackdropBorderColor())
 
 PinSettingsAppearanceFontColor.tag = editing_note.text
 PinSettingsAppearanceFontColor:update(editing_note.text:GetTextColor())

 PinSettingsAppearanceNoteWidth:SetValue(editing_note.maxWidth)
 PinSettingsAppearancePinWidth:SetValue(editing_note.pin:GetWidth())
 PinSettingsAppearancePinHeight:SetValue(editing_note.pin:GetHeight())
 PinSettingsAppearanceFontSize:SetValue(select(2, editing_note.text:GetFont()))

 -- general
 PinSettingsGeneralCombatShow:SetChecked(editing_note.combat_behavior == 0)
 PinSettingsGeneralCombatHide:SetChecked(editing_note.combat_behavior == 1)
 PinSettingsGeneralCombatClickThrough:SetChecked(editing_note.combat_behavior == 2)
 
 -- reminder
 PinSettingsReminderEnabled:SetChecked(editing_note.reminder_enabled)
 if editing_note.reminder_type == 1 then
  PinSettingsReminderRepeat:Click()
  reminder_type_tab_click(PinSettingsReminderRepeat)
 else
  PinSettingsReminderOneDay:Click()
  reminder_type_tab_click(PinSettingsReminderOneDay)
 end
 PinSettingsReminderSetHour:SetValue(editing_note.reminder_hour or 0)
 PinSettingsReminderSetMinute:SetValue(editing_note.reminder_minute or 0)
 
 for i = 1,7 do
  _G["PinSettingsReminderWeekday"..i]:SetChecked(editing_note["reminder_weekday"..i])
 end
 
 reminder_time_refresh()
 reminder_day_refresh()
end

function settings:EditNote(note)
 if not initialized then 
  Initialize()
 elseif mf:IsVisible() then
  mf:Hide() -- force onhide script to update other notes
 end

 mf:Show()
 
 NoteToFrame(note)
 reminder_tab_refresh()
end

local menuframe = CreateFrame("Frame", "PinMenuFrame", UIParent, "UIDropDownMenuTemplate")
local settings_menu = {
                       { text = SETTINGS, notCheckable = true, func = function() settings:EditNote(editing_note) end },
					   { text = " ", isTitle = true, notCheckable = true}, 
					   --{ text = "Delete", notCheckable = true, func = function() notes:Delete(editing_note) end },
					   --{ text = " ", isTitle = true, notCheckable = true}, 
					   { text = CANCEL, notCheckable = true}, 
                      }

function Pin_Settings(note)
 editing_note = note
 EasyMenu(settings_menu, menuframe, note.pin, 0, 0)
end




